From: Alexandre Chartre <alexandre.chartre@oracle.com>
To: linux-kernel@vger.kernel.org, mingo@kernel.org,
jpoimboe@kernel.org, peterz@infradead.org
Cc: alexandre.chartre@oracle.com
Subject: [PATCH v3 20/28] objtool: Disassemble exception table alternatives
Date: Wed, 12 Nov 2025 17:03:07 +0100 [thread overview]
Message-ID: <20251112160315.2207947-21-alexandre.chartre@oracle.com> (raw)
In-Reply-To: <20251112160315.2207947-1-alexandre.chartre@oracle.com>
When using the --disas option, also disable exception tables (EX_TABLE)
Signed-off-by: Alexandre Chartre <alexandre.chartre@oracle.com>
---
tools/objtool/disas.c | 45 +++++++++++++++++++++++++++++++++++++++----
1 file changed, 41 insertions(+), 4 deletions(-)
diff --git a/tools/objtool/disas.c b/tools/objtool/disas.c
index dabf2340e9ccf..42f3bf310b033 100644
--- a/tools/objtool/disas.c
+++ b/tools/objtool/disas.c
@@ -535,13 +535,26 @@ static int disas_alt_init(struct disas_alt *dalt,
struct alternative *alt,
int alt_num)
{
+ char *str;
+
dalt->orig_insn = orig_insn;
dalt->alt = alt;
dalt->insn_idx = 0;
- dalt->name = alt ? strfmt("ALTERNATIVE %d", alt_num) :
- strfmt("<alternative.%lx>", orig_insn->offset);
- if (!dalt->name)
+ if (!alt) {
+ str = strfmt("<alternative.%lx>", orig_insn->offset);
+ } else {
+ switch (alt->type) {
+ case ALT_TYPE_EX_TABLE:
+ str = strdup("EXCEPTION");
+ break;
+ default:
+ str = strfmt("ALTERNATIVE %d", alt_num);
+ break;
+ }
+ }
+ if (!str)
return -1;
+ dalt->name = str;
dalt->width = strlen(dalt->name);
return 0;
@@ -567,6 +580,26 @@ static int disas_alt_add_insn(struct disas_alt *dalt, int index, char *insn_str,
return 0;
}
+/*
+ * Disassemble an exception table alternative.
+ */
+static int disas_alt_extable(struct disas_alt *dalt)
+{
+ struct instruction *alt_insn;
+ char *str;
+
+ alt_insn = dalt->alt->insn;
+ str = strfmt("resume at 0x%lx <%s+0x%lx>",
+ alt_insn->offset, alt_insn->sym->name,
+ alt_insn->offset - alt_insn->sym->offset);
+ if (!str)
+ return -1;
+
+ disas_alt_add_insn(dalt, 0, str, 0);
+
+ return 1;
+}
+
/*
* Disassemble an alternative and store instructions in the disas_alt
* structure. Return the number of instructions in the alternative.
@@ -730,12 +763,16 @@ static void *disas_alt(struct disas_context *dctx,
goto error;
/*
* Only group alternatives are supported at the moment.
*/
switch (dalt->alt->type) {
case ALT_TYPE_INSTRUCTIONS:
count = disas_alt_group(dctx, dalt);
break;
+ case ALT_TYPE_EX_TABLE:
+ count = disas_alt_extable(dalt);
+ break;
default:
count = 0;
}
@@ -730,12 +763,16 @@ static void *disas_alt(struct disas_context *dctx,
goto error;
/*
- * Only group alternatives are supported at the moment.
+ * Only group alternatives and exception tables are
+ * supported at the moment.
*/
switch (dalt->alt->type) {
--
2.43.5
next prev parent reply other threads:[~2025-11-12 16:04 UTC|newest]
Thread overview: 32+ messages / expand[flat|nested] mbox.gz Atom feed top
2025-11-12 16:02 [PATCH v3 00/28] objtool: Function validation tracing Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 01/28] objtool: Move disassembly functions to a separated file Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 02/28] objtool: Create disassembly context Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 03/28] objtool: Disassemble code with libopcodes instead of running objdump Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 04/28] tool build: Remove annoying newline in build output Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 05/28] objtool: Print symbol during disassembly Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 06/28] objtool: Store instruction disassembly result Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 07/28] objtool: Disassemble instruction on warning or backtrace Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 08/28] objtool: Extract code to validate instruction from the validate branch loop Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 09/28] objtool: Record symbol name max length Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 10/28] objtool: Add option to trace function validation Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 11/28] objtool: Trace instruction state changes during " Alexandre Chartre
2025-11-12 16:02 ` [PATCH v3 12/28] objtool: Improve register reporting " Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 13/28] objtool: Identify the different types of alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 14/28] objtool: Improve tracing of alternative instructions Alexandre Chartre
2025-11-13 5:14 ` kernel test robot
2025-11-12 16:03 ` [PATCH v3 15/28] objtool: Do not validate IBT for .return_sites and .call_sites Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 16/28] objtool: Add the --disas=<function-pattern> action Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 17/28] objtool: Print headers for alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 18/28] objtool: Disassemble group alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 19/28] objtool: Print addresses with alternative instructions Alexandre Chartre
2025-11-12 16:03 ` Alexandre Chartre [this message]
2025-11-12 16:03 ` [PATCH v3 21/28] objtool: Disassemble jump table alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 22/28] objtool: Fix address references in alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 23/28] objtool: Provide access to feature and flags of group alternatives Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 24/28] objtool: Function to get the name of a CPU feature Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 25/28] objtool: Improve naming of group alternatives Alexandre Chartre
2025-11-13 5:15 ` kernel test robot
2025-11-12 16:03 ` [PATCH v3 26/28] objtool: Get the destination name of a PV call Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 27/28] objtool: Improve the disassembly of the pv_ops call Alexandre Chartre
2025-11-12 16:03 ` [PATCH v3 28/28] objtool: Print single line for alternatives with one instruction Alexandre Chartre
2025-11-13 9:21 ` [PATCH v3 00/28] objtool: Function validation tracing Alexandre Chartre
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20251112160315.2207947-21-alexandre.chartre@oracle.com \
--to=alexandre.chartre@oracle.com \
--cc=jpoimboe@kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@kernel.org \
--cc=peterz@infradead.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®